/* 
 * Copyright (c) 2002 Michal Moskal <malekith@pld-linux.org>
 * All rights reserved. See file COPYRIGHT for details about licensing.
 *
 * $Rev: 908 $ $LastChangedDate: 2002-09-08 01:32:56 +0200 (Sun, 08 Sep 2002) $
 *
 * Standard library for Gont, String module C implementation.
 */

/* This is for memrchr. */
#define _GNU_SOURCE

#include <gont/runtime.h>
#include <gont/array.h>
#include <string.h>

const char *String__to_cstring(struct gont_string *s)
{
	return s->data;
}

gont_ptroff String__length(struct gont_string *s)
{
	return s->len;
}

struct gont_string *String__of_cstring(const char *s)
{
	if (s == NULL)
		gont_invalid_argument("String::of_cstring");

	/* no gont_cstring() for efficiency */
	return gont_make_string(strlen(s), s);
}

struct gont_string *String__sub(struct gont_string *s, gont_ptroff pos, 
				gont_ptroff len)
{
	/* Note that first check isn't redundant (overflow). */
	if (pos > s->len || pos + len > s->len)
		gont_invalid_argument("String::sub");
		
	return gont_make_string(len, s->data + pos);
}


gont_ptroff String__get(struct gont_string *s, gont_ptroff idx)
{
	if (idx >= s->len)
		gont_invalid_argument("String::get");
		
	return (unsigned char)s->data[idx];
}

struct gont_string *String__make(gont_ptroff n, gont_ptroff c)
{
	struct gont_string *s = gont_make_string(n, NULL);
	memset(s->data, c, n);
	return s;
}

gont_ptroff String___index_from(struct gont_string *s, gont_ptroff pos, 
				gont_ptroff c)
{
	unsigned char *p;
	
	if (pos >= s->len)
		return -2;
		
	p = memchr(s->data + pos, c, s->len - pos);

	if (p)
		return p - s->data;
	else
		return -1;
}

gont_ptroff String___rindex_from(struct gont_string *s, gont_ptroff pos, 
				 gont_ptroff c)
{
	unsigned char *p;
	
	if (pos >= s->len)
		return -2;
		
	p = memrchr(s->data, c, pos + 1);

	if (p)
		return p - s->data;
	else
		return -1;
}

gont_ptroff String___index_str_from(struct gont_string *s, gont_ptroff pos, 
				    struct gont_string *c)
{
	unsigned char *p;
	int len;
	
	if (pos > s->len)
		return -2;
		
	if (c->len == 0)
		return pos;
	
	len = s->len - pos - c->len + 1;

	if (len < 0)
		return -1;
	
	for (p = s->data + pos; len--; p++)
		if (memcmp(p, c->data, c->len) == 0)
			return p - s->data;

	return -1;
}

struct gont_string *String__of_char_array(struct gont_array *src)
{
	struct gont_string *ret;
	gont_ptroff i;

	ret = gont_make_string(src->length, NULL);
	
	for (i = 0; i < src->length; i++)
		ret->data[i] = src->data[i];
	
	return ret;
}

struct gont_array *String__to_char_array(struct gont_string *src)
{
	struct gont_array *ret;
	gont_ptroff i;

	ret = Array___alloc(src->len);
	
	for (i = 0; i < src->len; i++)
		ret->data[i] = src->data[i];
	
	return ret;
}
