/* 
 * Copyright (c) 2002 Michal Moskal <malekith@pld-linux.org>
 * All rights reserved. See file COPYRIGHT for details about licensing.
 *
 * $Rev: 877 $ $LastChangedDate: 2002-09-07 15:34:12 +0200 (Sat, 07 Sep 2002) $
 *
 * Standard library for Gont, Bitvector module interface.
 */

/** Bitvector is in principle array of bools. However more space-efficient
    implementation is provided then <bool>Array::t.  Additionally standard
    bit-operations are available.  */

/** Type of bitvectors.  */
type t;

/** Return fresh bitvector, of [size] elements (bits), will set all of 
    them to [ini].  */
t make(int size, bool ini) = "Bitvector__make";

/** Return fresh bitvector, of [size] elements (bits). Elements are 
    initialized by calling [f(0)], [f(1)], ..., [f(size-1)] and storing 
    return values.  */
t init(int size, *(bool (int)) f) = "Bitvector__make";

/** Return value of element number [i] of bitvector [b].  Raise 
    [Invalid_argument] if [i] is outside allowed range 
    [0] -- [Bitvector::length(b) - 1].  */
bool get(t b, int i) = "Bitvector__get";

/** Set element number [i] of bitvector [b] to [v].  Raise [Invalid_argument] 
    if [i] is outside allowed range [0] -- [Bitvector::length(b) - 1].  */
void set(t b, int i, bool v) = "Bitvector__set";

/** Return number of elements (bits) in bitvector [b].  */
int length(t b) = "Bitvector__length";

/** Resize [b] to be [new_size] bit long.  If bitvector grows, new elements
    are all initialized to [ini].  */
void resize(t b, int new_size, bool ini) = "Bitvector__resize";

/** Return fresh bitvector, initialized to contents of bitvector [src].  */
t copy(t src) = "Bitvector__copy";

/** Perform logical OR on bits of [dst] and [src].  Result is stored in [dst].
    If [dst] is smaller then [src], it is zero-padded.  If [src] is smaller 
    then [dst] it is virtually zero-padded (it is not changed, but operation 
    is performed as if it was grown with zeros).  */
void or(t dst, t src);

/** Same as Bitvector::or, but performs logical AND.  */
void and(t dst, t src);

/** Same as Bitvector::or, but performs logical XOR.  */
void xor(t dst, t src);
